@@ -1,13 +1,17 @@
 -- Arbitrary.hs: QuickCheck instances
--- Copyright © 2014  Clint Adams
+-- Copyright © 2014-2015  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
 module Codec.Encryption.OpenPGP.Arbitrary () where
 
 import Codec.Encryption.OpenPGP.Types
-import qualified Data.ByteString as B
-import Test.QuickCheck (Arbitrary(..), choose, elements, frequency, getPositive, oneof, vector)
+import Control.Monad (liftM)
+import qualified Data.ByteString.Lazy as BL
+import qualified Data.List.NonEmpty as NE
+import Data.Maybe (fromMaybe)
+import Network.URI (nullURI, parseURI)
+import Test.QuickCheck (Arbitrary(..), choose, elements, frequency, getPositive, listOf1, oneof, vector)
 import Test.QuickCheck.Instances ()
 
 instance Arbitrary PKESK where
@@ -18,10 +22,10 @@ instance Arbitrary PKESK where
                    return $ PKESK pv eoki pka mpis
 
 instance Arbitrary Signature where
-    arbitrary = arbitrary >>= return . Signature
+    arbitrary = liftM Signature arbitrary
 
 instance Arbitrary UserId where
-    arbitrary = arbitrary >>= return . UserId
+    arbitrary = liftM UserId arbitrary
 
 --
 
@@ -60,28 +64,28 @@ instance Arbitrary SigSubPacket where
 instance Arbitrary SigSubPacketPayload where
     arbitrary = oneof [sct, set, ec, ts, re, ket, psa, rk, i, nd, phas, pcas, ksps, pks, puid, purl, kfs, suid, rfr, fs, st {-, es -}, udss, oss]
         where
-            sct = arbitrary >>= return . SigCreationTime
-            set = arbitrary >>= return . SigExpirationTime
-            ec = arbitrary >>= return . ExportableCertification
+            sct = liftM SigCreationTime arbitrary
+            set = liftM SigExpirationTime arbitrary
+            ec = liftM ExportableCertification arbitrary
             ts = arbitrary >>= \tl -> arbitrary >>= \ta -> return (TrustSignature tl ta)
-            re = arbitrary >>= return . RegularExpression
-            ket = arbitrary >>= return . KeyExpirationTime
-            psa = arbitrary >>= return . PreferredSymmetricAlgorithms
+            re = liftM RegularExpression arbitrary
+            ket = liftM KeyExpirationTime arbitrary
+            psa = liftM PreferredSymmetricAlgorithms arbitrary
             rk = arbitrary >>= \rcs -> arbitrary >>= \pka -> arbitrary >>= \tof -> return (RevocationKey rcs pka tof)
-            i = arbitrary >>= return . Issuer
+            i = liftM Issuer arbitrary
             nd = arbitrary >>= \nfs -> arbitrary >>= \nn -> arbitrary >>= \nv -> return (NotationData nfs nn nv)
-            phas = arbitrary >>= return . PreferredHashAlgorithms
-            pcas = arbitrary >>= return . PreferredCompressionAlgorithms
-            ksps = arbitrary >>= return . KeyServerPreferences
-            pks = arbitrary >>= return . PreferredKeyServer
-            puid = arbitrary >>= return . PrimaryUserId
-            purl = arbitrary >>= return . PolicyURL
-            kfs = arbitrary >>= return . KeyFlags
-            suid = arbitrary >>= return . SignersUserId
+            phas = liftM PreferredHashAlgorithms arbitrary
+            pcas = liftM PreferredCompressionAlgorithms arbitrary
+            ksps = liftM KeyServerPreferences arbitrary
+            pks = liftM PreferredKeyServer arbitrary
+            puid = liftM PrimaryUserId arbitrary
+            purl = liftM (PolicyURL . URL . fromMaybe nullURI . parseURI) arbitrary
+            kfs = liftM KeyFlags arbitrary
+            suid = liftM SignersUserId arbitrary
             rfr = arbitrary >>= \rc -> arbitrary >>= \rr -> return (ReasonForRevocation rc rr)
-            fs = arbitrary >>= return . Features
+            fs = liftM Features arbitrary
             st = arbitrary >>= \pka -> arbitrary >>= \ha -> arbitrary >>= \sh -> return (SignatureTarget pka ha sh)
-            es = arbitrary >>= return . EmbeddedSignature -- FIXME: figure out why EmbeddedSignature fails to serialize properly
+            es = liftM EmbeddedSignature arbitrary -- FIXME: figure out why EmbeddedSignature fails to serialize properly
             udss = choose (100,110) >>= \a -> arbitrary >>= \b -> return (UserDefinedSigSub a b)
             oss = choose (111,127) >>= \a -> arbitrary >>= \b -> return (OtherSigSub a b) -- FIXME: more comprehensive range
 
@@ -91,13 +95,13 @@ instance Arbitrary PubKeyAlgorithm where
     arbitrary = elements [RSA, DSA, ECDH, ECDSA, DH]
 
 instance Arbitrary EightOctetKeyId where
-    arbitrary = vector 8 >>= return . EightOctetKeyId . B.pack
+    arbitrary = liftM (EightOctetKeyId . BL.pack) (vector 8)
 
 instance Arbitrary TwentyOctetFingerprint where
-    arbitrary = vector 20 >>= return . TwentyOctetFingerprint . B.pack
+    arbitrary = liftM (TwentyOctetFingerprint . BL.pack) (vector 20)
 
 instance Arbitrary MPI where
-    arbitrary = arbitrary >>= return . MPI . getPositive
+    arbitrary = liftM (MPI . getPositive) arbitrary
 
 instance Arbitrary SigType where
     arbitrary = elements [BinarySig, CanonicalTextSig, StandaloneSig, GenericCert, PersonaCert, CasualCert, PositiveCert, SubkeyBindingSig, PrimaryKeyBindingSig, SignatureDirectlyOnAKey, KeyRevocationSig, SubkeyRevocationSig, CertRevocationSig, TimestampSig, ThirdPartyConfirmationSig]
@@ -112,13 +116,13 @@ instance Arbitrary RevocationClass where
     arbitrary = frequency [(9,srk),(1,rco)]
         where
             srk = return SensitiveRK
-            rco = choose (2,7) >>= return . RClOther
+            rco = liftM RClOther (choose (2,7))
 
 instance Arbitrary NotationFlag where
     arbitrary = frequency [(9,hr),(1,onf)]
         where
             hr = return HumanReadable
-            onf = choose (1,31) >>= return . OtherNF
+            onf = liftM OtherNF (choose (1,31))
 
 instance Arbitrary CompressionAlgorithm where
     arbitrary = elements [Uncompressed,ZIP,ZLIB,BZip2]
@@ -127,7 +131,7 @@ instance Arbitrary KSPFlag where
     arbitrary = frequency [(9,nm),(1,kspo)]
         where
             nm = return NoModify
-            kspo = choose (2,63) >>= return . KSPOther
+            kspo = liftM KSPOther (choose (2,63))
 
 instance Arbitrary KeyFlag where
     arbitrary = elements [GroupKey, AuthKey, SplitKey, EncryptStorageKey, EncryptCommunicationsKey, SignDataKey, CertifyKeysKey]
@@ -139,4 +143,14 @@ instance Arbitrary FeatureFlag where
     arbitrary = frequency [(9,md),(1,fo)]
         where
             md = return ModificationDetection
-            fo = choose (8,63) >>= return . FeatureOther
+            fo = liftM FeatureOther (choose (8,63))
+
+instance Arbitrary ThirtyTwoBitTimeStamp where
+    arbitrary = liftM ThirtyTwoBitTimeStamp arbitrary
+
+instance Arbitrary ThirtyTwoBitDuration where
+    arbitrary = liftM ThirtyTwoBitDuration arbitrary
+
+-- FIXME: this should be elsewhere
+instance Arbitrary a => Arbitrary (NE.NonEmpty a) where
+    arbitrary = NE.fromList `liftM` listOf1 arbitrary
