@@ -1,5 +1,5 @@
 -- Signatures.hs: OpenPGP (RFC4880) signature verification
--- Copyright © 2012-2014  Clint Adams
+-- Copyright © 2012-2015  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -19,14 +19,17 @@ import Crypto.PubKey.HashDescr (HashDescr(..))
 import qualified Crypto.PubKey.DSA as DSA
 import qualified Crypto.PubKey.RSA.PKCS15 as P15
 
-import Data.ByteString (ByteString)
+import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString as B
+import qualified Data.ByteString.Lazy as BL
 import Data.Either (lefts, rights)
 import Data.IxSet ((@=))
 import qualified Data.IxSet as IxSet
+import Data.List.NonEmpty (NonEmpty(..))
+import Data.Text (Text)
 import Data.Time.Clock (UTCTime(..), diffUTCTime)
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
-import Data.Serialize.Put (runPut)
+import Data.Binary.Put (runPut)
 
 import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID, fingerprint)
 import Codec.Encryption.OpenPGP.Internal (countBits, integerToBEBS, PktStreamContext(..), issuer, emptyPSC, hashDescr)
@@ -58,7 +61,7 @@ verifyTKWith vsf mt key = do
         checkRevokers = Right . concat . rights . map verifyRevoker . filter isRevokerP . _tkRevs
         checkKeyRevocations :: [(PubKeyAlgorithm, TwentyOctetFingerprint)] -> TK -> Either String [SignaturePayload]
         checkKeyRevocations rs k = Prelude.sequence . concatMap (filterRevs rs) . rights . map (liftM2 fmap (,) vSig) $ k^.tkRevs
-        checkUidSigs :: [(String, [SignaturePayload])] -> [(String, [SignaturePayload])]
+        checkUidSigs :: [(Text, [SignaturePayload])] -> [(Text, [SignaturePayload])]
         checkUidSigs = map (\(uid, sps) -> (uid, (rights . map (\sp -> fmap (const sp) (vUid (uid, sp)))) sps))
         checkUAtSigs :: [([UserAttrSubPacket], [SignaturePayload])] -> [([UserAttrSubPacket], [SignaturePayload])]
         checkUAtSigs = map (\(uat, sps) -> (uat, (rights . map (\sp -> fmap (const sp) (vUAt (uat, sp)))) sps))
@@ -71,13 +74,11 @@ verifyTKWith vsf mt key = do
         checkSub' p sps = let goodsigs = filter (vSubSig p) (filter isSubkeyBindingSig sps) in if null goodsigs then [] else [(p, goodsigs)]
         getHasheds (SigV4 _ _ _ ha _ _ _) = ha
         getHasheds _ = []
-	filterRevs :: [(PubKeyAlgorithm, TwentyOctetFingerprint)] -> (SignaturePayload, Verification) -> [Either String SignaturePayload]
-	filterRevs vokers spv = case spv of
+        filterRevs :: [(PubKeyAlgorithm, TwentyOctetFingerprint)] -> (SignaturePayload, Verification) -> [Either String SignaturePayload]
+        filterRevs vokers spv = case spv of
                                      (s@(SigV4 SignatureDirectlyOnAKey _ _ _ _ _ _), _) -> [Right s]
                                      (s@(SigV4 KeyRevocationSig pka _ _ _ _ _), v) -> if (v^.verificationSigner == key ^. tkKey._1) || any (\(p,f) -> p == pka && f == fingerprint (v^.verificationSigner)) vokers then [Left "Key revoked"] else [Right s]
-				     _ -> []
-        isKeyRevocation (SigV4 KeyRevocationSig _ _ _ _ _ _) = True
-        isKeyRevocation _ = False
+                                     _ -> []
         isSubkeyRevocation (SigV4 SubkeyRevocationSig _ _ _ _ _ _) = True
         isSubkeyRevocation _ = False
         isSubkeyBindingSig (SigV4 SubkeyBindingSig _ _ _ _ _ _) = True
@@ -88,7 +89,7 @@ verifyTKWith vsf mt key = do
         isRevocationKeySSP _ = False
         isIssuerSSP (SigSubPacket _ (Issuer _)) = True
         isIssuerSSP _ = False
-        vUid :: (String, SignaturePayload) -> Either String Verification
+        vUid :: (Text, SignaturePayload) -> Either String Verification
         vUid (uid, sp) = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1), lastUIDorUAt = UserIdPkt uid } mt
         vUAt :: ([UserAttrSubPacket], SignaturePayload) -> Either String Verification
         vUAt (uat, sp) = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1), lastUIDorUAt = UserAttributePkt uat } mt
@@ -97,7 +98,7 @@ verifyTKWith vsf mt key = do
         vSubSig :: Pkt -> SignaturePayload -> Bool
         vSubSig sk sp = case vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1), lastSubkey = sk} mt of
                                 Left _ -> False
-				Right _ -> True
+                                Right _ -> True
         verifyRevoker :: SignaturePayload -> Either String [(PubKeyAlgorithm, TwentyOctetFingerprint)]
         verifyRevoker sp = do
             _ <- vSig sp
@@ -112,47 +113,46 @@ verifyAgainstKeyring kr sig mt payload = do
 verifyAgainstKeys :: [TK] -> Pkt -> Maybe UTCTime -> ByteString -> Either String Verification
 verifyAgainstKeys ks sig mt payload = do
     let allrelevantpkps = filter (\x -> ((==) <$> issuer sig <*> hush (eightOctetKeyID x)) == Just True) (concatMap (\x -> (x ^. tkKey._1):map subPKP (_tkSubs x)) ks)
-    let results = map (\pkp -> verify' sig pkp (hashalgo sig) (finalPayload sig payload)) allrelevantpkps
+    let results = map (\pkp -> verify' sig pkp (hashalgo sig) (BL.toStrict (finalPayload sig payload))) allrelevantpkps
     case rights results of
         [] -> Left (concatMap (++"/") (lefts results))
         [r] -> do _ <- isSignatureExpired sig mt
-	          return (Verification r ((_signaturePayload . fromPkt) sig)) -- FIXME: this should also check expiration time and flags of the signing key
+                  return (Verification r ((_signaturePayload . fromPkt) sig)) -- FIXME: this should also check expiration time and flags of the signing key
         _ -> Left "multiple successes; unexpected condition"
     where
         subPKP (pack, _) = subPKP' pack
         subPKP' (PublicSubkeyPkt p) = p
         subPKP' (SecretSubkeyPkt p _) = p
-        verify' (SignaturePkt s) (pub@(PKPayload V4 _ _ _ pkey)) ha pl = verify'' (pkaAndMPIs s) ha pub pkey pl
+        subPKP' _ = error "This should never happen (subPKP')"
+        verify' (SignaturePkt s) (pub@(PKPayload V4 _ _ _ pkey)) ha pl = hashDescr ha >>= \hd -> verify'' (pkaAndMPIs s) hd pub pkey pl
         verify' _ _ _ _ = error "This should never happen (verify')."
-        verify'' (DSA,mpis) ha pub (DSAPubKey pkey) bs = verify''' (dsaVerify mpis (hashDescr ha) pkey bs) pub
-        verify'' (RSA,mpis) ha pub (RSAPubKey pkey) bs = verify''' (rsaVerify mpis (hashDescr ha) pkey bs) pub
+        verify'' (DSA,mpis) hd pub (DSAPubKey (DSA_PublicKey pkey)) bs = verify''' (dsaVerify mpis hd pkey bs) pub
+        verify'' (RSA,mpis) hd pub (RSAPubKey (RSA_PublicKey pkey)) bs = verify''' (rsaVerify mpis hd pkey bs) pub
         verify'' _ _ _ _ _ = Left "unimplemented key type"
-	verify''' f pub = if f then Right pub else Left "verification failed"
-	dsaVerify (r:s:[]) (Right hd) pkey = DSA.verify (dsaTruncate pkey . hashFunction hd) pkey (dsaMPIsToSig r s)
-	dsaVerify _ (Right _) _ = const False -- FIXME: this should be some sort of Either chain
-	dsaVerify _ (Left _) _ = const False  -- FIXME: this should be some sort of Either chain
-	rsaVerify mpis (Right hd) pkey bs = P15.verify hd pkey bs (rsaMPItoSig mpis)
-	rsaVerify _ (Left _) _ _ = False  -- FIXME: this should be some sort of Either chain
+        verify''' f pub = if f then Right pub else Left "verification failed"
+        dsaVerify (r:|[s]) hd pkey = DSA.verify (dsaTruncate pkey . hashFunction hd) pkey (dsaMPIsToSig r s)
+        dsaVerify _ _ _ = const False -- FIXME: this should be some sort of Either chain?
+        rsaVerify mpis hd pkey bs = P15.verify hd pkey bs (BL.toStrict (rsaMPItoSig mpis))
         dsaMPIsToSig r s = DSA.Signature (unMPI r) (unMPI s)
-        rsaMPItoSig mpis = integerToBEBS (unMPI (head mpis))
+        rsaMPItoSig (sig:|[]) = integerToBEBS (unMPI sig)
         hashalgo :: Pkt -> HashAlgorithm
         hashalgo (SignaturePkt (SigV4 _ _ ha _ _ _ _)) = ha
         hashalgo _ = error "This should never happen (hashalgo)."
-        dsaTruncate pkey bs = if countBits bs > dsaQLen pkey then B.take (fromIntegral (dsaQLen pkey) `div` 8) bs else bs -- FIXME: uneven bits
+        dsaTruncate pkey bs = let lbs = BL.fromStrict bs in if countBits lbs > dsaQLen pkey then B.take (fromIntegral (dsaQLen pkey) `div` 8) bs else bs -- FIXME: uneven bits
         dsaQLen = countBits . integerToBEBS . DSA.params_q . DSA.public_params
-	pkaAndMPIs (SigV4 _ pka _ _ _ _ mpis) = (pka,mpis)
-	pkaAndMPIs _ = error "This should never happen (pkaAndMPIs)."
+        pkaAndMPIs (SigV4 _ pka _ _ _ _ mpis) = (pka,mpis)
+        pkaAndMPIs _ = error "This should never happen (pkaAndMPIs)."
         isSignatureExpired :: Pkt -> Maybe UTCTime -> Either String Bool
-        isSignatureExpired s Nothing = return False
+        isSignatureExpired _ Nothing = return False
         isSignatureExpired s (Just t) = if any (expiredBefore t) ((\(SigV4 _ _ _ h _ _ _) -> h) . _signaturePayload . fromPkt $ s) then Left "signature expired" else return True
         expiredBefore :: UTCTime -> SigSubPacket -> Bool
         expiredBefore ct (SigSubPacket _ (SigExpirationTime et)) = fromEnum ((posixSecondsToUTCTime . toEnum . fromEnum) et `diffUTCTime` ct) < 0
         expiredBefore _ _ = False
 
 finalPayload :: Pkt -> ByteString -> ByteString
-finalPayload s pl = B.concat [pl, sigbit s, trailer s]
+finalPayload s pl = BL.concat [pl, sigbit, trailer s]
     where
-        sigbit s = runPut $ putPartialSigforSigning s
+        sigbit = runPut $ putPartialSigforSigning s
         trailer :: Pkt -> ByteString
-        trailer s@(SignaturePkt (SigV4 {})) = runPut $ putSigTrailer s
-        trailer _ = B.empty
+        trailer (SignaturePkt (SigV4 {})) = runPut $ putSigTrailer s
+        trailer _ = BL.empty
