@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 
 -- suite.hs: hOpenPGP test suite
--- Copyright © 2012-2014  Clint Adams
+-- Copyright © 2012-2015  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -9,6 +9,7 @@ import Test.Tasty (defaultMain, testGroup, TestTree)
 import Test.Tasty.HUnit (testCase, Assertion, assertFailure, assertEqual)
 import Test.Tasty.QuickCheck as QC
 
+import Data.Bifunctor (bimap)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import Codec.Encryption.OpenPGP.Arbitrary ()
@@ -23,26 +24,31 @@ import Codec.Encryption.OpenPGP.Types
 import Control.Error.Util (isRight)
 import Control.Monad.Trans.Resource (ResourceT, runResourceT)
 import Crypto.PubKey.RSA (PrivateKey(private_pub))
-import Data.Conduit.Cereal (conduitGet)
+import Data.Conduit.Serialization.Binary (conduitGet)
 import Data.Conduit.OpenPGP.Compression (conduitCompress, conduitDecompress)
 import Data.Conduit.OpenPGP.Decrypt (conduitDecrypt)
 import Data.Conduit.OpenPGP.Keyring (conduitToTKs, conduitToTKsDropping, sinkKeyringMap)
 import Data.Conduit.OpenPGP.Verify (conduitVerify)
 import Data.IxSet ((@=), getOne)
 import Data.Maybe (isJust)
-import Data.Serialize (get, put)
-import Data.Serialize.Get (runGet, Get)
-import Data.Serialize.Put (runPut)
+import Data.Binary (get, put)
+import Data.Binary.Get (runGetOrFail, Get)
+import Data.Binary.Put (runPut)
 import Data.Text (Text)
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
+import Text.PrettyPrint.Free (pretty)
 
 import qualified Data.Conduit as DC
 import qualified Data.Conduit.Binary as CB
 import qualified Data.Conduit.List as CL
 
+-- this needs a better name
+runGet :: Get a -> BL.ByteString -> Either String a
+runGet g bs = bimap (\(_,_,x) -> x) (\(_,_,x) -> x) (runGetOrFail g bs)
+
 testSerialization :: FilePath -> Assertion
 testSerialization fpr = do
-    bs <- B.readFile $ "tests/data/" ++ fpr
+    bs <- BL.readFile $ "tests/data/" ++ fpr
     let firstpass = runGet get bs
     case fmap unBlock firstpass of
         Left _ -> assertFailure $ "First pass failed on " ++ fpr
@@ -56,7 +62,7 @@ testSerialization fpr = do
 
 testCompression :: FilePath -> Assertion
 testCompression fpr = do
-    bs <- B.readFile $ "tests/data/" ++ fpr
+    bs <- BL.readFile $ "tests/data/" ++ fpr
     let firstpass = fmap (concatMap decompressPkt . unBlock) . runGet get $ bs
     case firstpass of
         Left _ -> assertFailure $ "First pass failed on " ++ fpr
@@ -78,12 +84,12 @@ testConduitOutputLength fpr c target = do
 
 testKeyIDandFingerprint :: FilePath -> String -> Assertion
 testKeyIDandFingerprint fpr kf = do
-    bs <- B.readFile $ "tests/data/" ++ fpr
+    bs <- BL.readFile $ "tests/data/" ++ fpr
     case runGet (get :: Get Pkt) bs of
         Left _ -> assertFailure $ "Decoding of " ++ fpr ++ " broke."
         Right (PublicKeyPkt pkp) -> do
-	                               assertEqual ("for " ++ fpr ++ " (spaceless)") (spaceless kf) (either (const "unknown") show (eightOctetKeyID pkp) ++ "/" ++ show (fingerprint pkp))
-	                               assertEqual ("for " ++ fpr ++ " (spaced)") kf (either (const "unknown") show (eightOctetKeyID pkp) ++ "/" ++ show (SpacedFingerprint (fingerprint pkp)))
+	                               assertEqual ("for " ++ fpr ++ " (spaceless)") (spaceless kf) (either (const "unknown") (show . pretty) (eightOctetKeyID pkp) ++ "/" ++ show (pretty (fingerprint pkp)))
+	                               assertEqual ("for " ++ fpr ++ " (spaced)") kf (either (const "unknown") (show . pretty) (eightOctetKeyID pkp) ++ "/" ++ show (pretty (SpacedFingerprint (fingerprint pkp))))
         _ -> assertFailure "Expected public key, got something else."
     where
         spaceless = filter (/=' ')
@@ -115,7 +121,7 @@ testKeysExpiration expectsuccess keyfile = do
     assertEqual (keyfile ++ " key expiration") expectsuccess tvalid
 
 -- This needs a lot of work
-testSymmetricEncryption :: FilePath -> FilePath -> B.ByteString -> Assertion
+testSymmetricEncryption :: FilePath -> FilePath -> BL.ByteString -> Assertion
 testSymmetricEncryption encfile passfile cleartext = do
   passphrase <- BL.readFile $ "tests/data/" ++ passfile
   -- get parse tree
@@ -141,8 +147,8 @@ testSecretKeyDecryption keyfile passfile = do
   passphrase <- BL.readFile $ "tests/data/" ++ passfile
   kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
   let SecretKey pkp ska = fromPkt . head $ kr
-      SUUnencrypted (RSAPrivateKey dpk) _ = decryptPrivateKey (pkp, ska) passphrase  -- FIXME: better API for multiple keytypes
-      RSAPubKey pk = _pubkey pkp
+      SUUnencrypted (RSAPrivateKey (RSA_PrivateKey dpk)) _ = decryptPrivateKey (pkp, ska) passphrase  -- FIXME: better API for multiple keytypes
+      RSAPubKey (RSA_PublicKey pk) = _pubkey pkp
   assertEqual "private key matches public key" pk (private_pub dpk)
 
 -- FIXME: this should be reworked either with tasty-golden or some other form of sanity
@@ -152,7 +158,7 @@ testSecretKeyEncryption keyfile passfile = do
   kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
   gkr <- runResourceT $ CB.sourceFile ("tests/data/" ++ "aes256-sha512.seckey") DC.$= conduitGet get DC.$$ CL.consume
   let SecretKey pkp ska = fromPkt . head $ kr
-      newska = encryptPrivateKey "\226~\197\a\202#\"G" "\187\219\253I\236\204\t5D\196\NAK>;\202\185\t" ska passphrase
+      newska = encryptPrivateKey "\226~\197\a\202#\"G" (IV "\187\219\253I\236\204\t5D\196\NAK>;\202\185\t") ska passphrase
       newtruck = toPkt (SecretKey pkp newska):tail kr
   assertEqual "encrypted private key matches golden file" gkr newtruck
 
@@ -255,7 +261,7 @@ unitTests = testGroup "Unit Tests" [
    , testCase "gnu-dummy-s2k-101-secret-key.gpg" (testSerialization "gnu-dummy-s2k-101-secret-key.gpg")
    ],
   testGroup "KeyID/fingerprint group" [
-     testCase "v3 key" (testKeyIDandFingerprint "v3.key" "C7261095/CBD9 F412 6807 E405 CC2D  2712 1DF5 E86E  ")
+     testCase "v3 key" (testKeyIDandFingerprint "v3.key" "C7261095/CBD9 F412 6807 E405 CC2D  2712 1DF5 E86E")
    , testCase "v4 key" (testKeyIDandFingerprint "000001-006.public_key" "D4D54EA16F87040E/421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E")
    ],
   testGroup "Keyring group" [
@@ -265,6 +271,7 @@ unitTests = testGroup "Unit Tests" [
    , testCase "secring 7732CF988A63EA86" (testKeyringLookup "secring.gpg" "7732CF988A63EA86" True)
    , testCase "secring 123456789ABCDEF0" (testKeyringLookup "secring.gpg" "123456789ABCDEF0" False)
    , testCase "secsub AD992E9C24399832" (testKeyringLookup "secring.gpg" "AD992E9C24399832" True)
+   , testCase "d-k 94FA372B2DA8B985" (testKeyringLookup "debian-keyring.gpg" "0x94FA372B2DA8B985" False)
    -- FIXME: should count keys in rings
    ],
   testGroup "Message verification group" [
